@@ -1,20 +1,15 @@
 import json
 import os
-import requests
 from copy import deepcopy
 from time import sleep
+
+import requests
+
+from citrination_client.search import *
+from citrination_client.util.quote_finder import quote
+
 from pypif import pif
 from pypif.util.case import keys_to_snake_case
-from citrination_client.util.quote_finder import quote
-from citrination_client.search.pif.query.pif_query import PifQuery
-from citrination_client.search.pif.query.core.field_operation import FieldOperation
-from citrination_client.search.pif.query.core.filter import Filter
-from citrination_client.search.pif.query.core.property_query import PropertyQuery
-from citrination_client.search.pif.query.core.reference_query import ReferenceQuery
-from citrination_client.search.pif.query.core.system_query import SystemQuery
-from citrination_client.search.pif.query.chemical.chemical_field_operation import  ChemicalFieldOperation
-from citrination_client.search.pif.query.chemical.chemical_filter import ChemicalFilter
-from citrination_client.search.pif.result.pif_search_result import PifSearchResult
 
 
 class CitrinationClient(object):
@@ -31,53 +26,107 @@ class CitrinationClient(object):
         https://citrination.com. This should point to the full url of the site to access. For example, to access
         the STEEL site on citrination, use 'https://STEEL.citrination.com'.
         """
-        self.headers = {'X-API-Key': quote(api_key), 'Content-Type': 'application/json'}
+        self.headers = {'X-API-Key': quote(api_key), 'Content-Type': 'application/json', 'X-Citrination-API-Version': '1.0.0'}
         self.api_url = site + '/api'
         self.pif_search_url = self.api_url + '/search/pif_search'
+        self.pif_multi_search_url = self.api_url + '/search/pif_multi_search'
+        self.dataset_search_url = self.api_url + '/search/dataset'
 
-    def search(self, pif_query):
+    def search(self, pif_system_returning_query):
+        """
+        Run a PIF query against Citrination. This is just an alias for the pif_search method for backwards
+        compatibility.
+
+        :param pif_system_returning_query: :class:`PifSystemReturningQuery` to execute.
+        :return: :class:`PifSearchResult` object with the results of the query.
+        """
+        return self.pif_search(pif_system_returning_query)
+
+    def pif_search(self, pif_system_returning_query):
         """
         Run a PIF query against Citrination.
 
-        :param pif_query: :class:`.PifQuery` to execute.
-        :return: :class:`.PifSearchResult` object with the results of the query.
+        :param pif_system_returning_query: :class:`PifSystemReturningQuery` to execute.
+        :return: :class:`PifSearchResult` object with the results of the query.
         """
-        if pif_query.size is None and pif_query.from_index is None:
+        if pif_system_returning_query.size is None and pif_system_returning_query.from_index is None:
             total = 1; time = 0.0; hits = []; first = True
             while len(hits) < min(total, 10000):
                 if first:
                     first = False
                 else:
                     sleep(3)
-                sub_query = deepcopy(pif_query)
+                sub_query = deepcopy(pif_system_returning_query)
                 sub_query.from_index = len(hits)
-                partial_results = self.search(sub_query)
+                partial_results = self.pif_search(sub_query)
                 total = partial_results.total_num_hits
                 time += partial_results.took
                 if partial_results.hits is not None:
                     hits.extend(partial_results.hits)
             return PifSearchResult(hits=hits, total_num_hits=total, took=time)
 
-        response = requests.post(self.pif_search_url, data=pif.dumps(pif_query), headers=self.headers)
+        response = self._post_with_version_check(self.pif_search_url, data=pif.dumps(pif_system_returning_query), headers=self.headers)
         if response.status_code != requests.codes.ok:
             raise RuntimeError('Received ' + str(response.status_code) + ' response: ' + str(response.reason))
         return PifSearchResult(**keys_to_snake_case(response.json()['results']))
 
+    def pif_multi_search(self, multi_query):
+        """
+        Run each in a list of PIF queries against Citrination.
+
+        :param multi_query: :class:`MultiQuery` object to execute.
+        :return: :class:`PifMultiSearchResult` object with the results of the query.
+        """
+        response = self._post_with_version_check(self.pif_multi_search_url, data=pif.dumps(multi_query), headers=self.headers)
+        if response.status_code != requests.codes.ok:
+            raise RuntimeError('Received ' + str(response.status_code) + ' response: ' + str(response.reason))
+        return PifMultiSearchResult(**keys_to_snake_case(response.json()['results']))
+
+    def dataset_search(self, dataset_returning_query):
+        """
+        Run a dataset query against Citrination.
+
+        :param dataset_returning_query: :class:`DatasetReturningQuery` to execute.
+        :return: :class:`DatasetSearchResult` object with the results of the query.
+        """
+        if dataset_returning_query.size is None and dataset_returning_query.from_index is None:
+            total = 1; time = 0.0; hits = []; first = True
+            while len(hits) < min(total, 10000):
+                if first:
+                    first = False
+                else:
+                    sleep(3)
+                sub_query = deepcopy(dataset_returning_query)
+                sub_query.from_index = len(hits)
+                partial_results = self.dataset_search(sub_query)
+                total = partial_results.total_num_hits
+                time += partial_results.took
+                if partial_results.hits is not None:
+                    hits.extend(partial_results.hits)
+            return DatasetSearchResult(hits=hits, total_num_hits=total, took=time)
+
+        response = self._post_with_version_check(
+            self.dataset_search_url, data=pif.dumps(dataset_returning_query), headers=self.headers)
+        if response.status_code != requests.codes.ok:
+            raise RuntimeError('Received ' + str(response.status_code) + ' response: ' + str(response.reason))
+        return DatasetSearchResult(**keys_to_snake_case(response.json()['results']))
+
     def simple_chemical_search(self, name=None, chemical_formula=None, property_name=None, property_value=None,
                                property_min=None, property_max=None, property_units=None, reference_doi=None,
                                include_datasets=[], exclude_datasets=[], from_index=None, size=None):
         """
-        Run a query against Citrination. This method generates a :class:`.PifQuery` object using the supplied arguments.
-        All arguments that accept lists have logical OR's on the queries that they generate. This means that, for
-        example, simple_chemical_search(name=['A', 'B']) will match records that have name equal to 'A' or 'B'.
+        Run a query against Citrination. This method generates a :class:`PifSystemReturningQuery` object using the
+        supplied arguments. All arguments that accept lists have logical OR's on the queries that they generate.
+        This means that, for example, simple_chemical_search(name=['A', 'B']) will match records that have name
+        equal to 'A' or 'B'.
 
-        Results will be pulled into the extracted field of the :class:`.PifSearchHit` objects that are returned. The
+        Results will be pulled into the extracted field of the :class:`PifSearchHit` objects that are returned. The
         name will appear under the key "name", chemical formula under "chemical_formula", property name under
         "property_name", value of the property under "property_value", units of the property under "property_units",
         and reference DOI under "reference_doi".
 
         This method is only meant for execution of very simple queries. More complex queries must use the search method
-        that accepts a :class:`.PifQuery` object.
+        that accepts a :class:`PifSystemReturningQuery` object.
 
         :param name: One or more strings with the names of the chemical system to match.
         :param chemical_formula:  One or more strings with the chemical formulas to match.
@@ -91,27 +140,27 @@ class CitrinationClient(object):
         :param exclude_datasets: One or more integers with dataset IDs that must not match.
         :param from_index: Index of the first record to match.
         :param size: Total number of records to return.
-        :return: :class:`.PifSearchResult` object with the results of the query.
+        :return: :class:`PifSearchResult` object with the results of the query.
         """
-        system_query = SystemQuery()
-        system_query.names = FieldOperation(
+        pif_system_query = PifSystemQuery()
+        pif_system_query.names = FieldQuery(
             extract_as='name',
             filter=[Filter(equal=i) for i in self._get_list(name)])
-        system_query.chemical_formula = ChemicalFieldOperation(
+        pif_system_query.chemical_formula = ChemicalFieldQuery(
             extract_as='chemical_formula',
             filter=[ChemicalFilter(equal=i) for i in self._get_list(chemical_formula)])
-        system_query.references = ReferenceQuery(doi=FieldOperation(
+        pif_system_query.references = ReferenceQuery(doi=FieldQuery(
             extract_as='reference_doi',
             filter=[Filter(equal=i) for i in self._get_list(reference_doi)]))
 
         # Generate the parts of the property query
-        property_name_query = FieldOperation(
+        property_name_query = FieldQuery(
             extract_as='property_name',
             filter=[Filter(equal=i) for i in self._get_list(property_name)])
-        property_units_query = FieldOperation(
+        property_units_query = FieldQuery(
             extract_as='property_units',
             filter=[Filter(equal=i) for i in self._get_list(property_units)])
-        property_value_query = FieldOperation(
+        property_value_query = FieldQuery(
             extract_as='property_value',
             filter=[])
         for i in self._get_list(property_value):
@@ -120,20 +169,27 @@ class CitrinationClient(object):
             property_value_query.filter.append(Filter(min=property_min, max=property_max))
 
         # Generate the full property query
-        system_query.properties = PropertyQuery(
+        pif_system_query.properties = PropertyQuery(
             name=property_name_query,
             value=property_value_query,
             units=property_units_query)
 
+        # Generate the dataset query
+        dataset_query = list()
+        if include_datasets:
+            dataset_query.append(DatasetQuery(logic='MUST', id=[Filter(equal=i) for i in include_datasets]))
+        if exclude_datasets:
+            dataset_query.append(DatasetQuery(logic='MUST_NOT', id=[Filter(equal=i) for i in exclude_datasets]))
+
         # Run the query
-        query = PifQuery(
-            system=system_query,
+        pif_system_returning_query = PifSystemReturningQuery(
+            query=DataQuery(
+                system=pif_system_query,
+                dataset=dataset_query),
             from_index=from_index,
             size=size,
-            score_relevance=True,
-            include_datasets=include_datasets,
-            exclude_datasets=exclude_datasets)
-        return self.search(query)
+            score_relevance=True)
+        return self.pif_search(pif_system_returning_query)
 
     @staticmethod
     def _get_list(values):
@@ -151,79 +207,225 @@ class CitrinationClient(object):
         else:
             return [values]
 
-    def predict(self, model_name, candidates):
+    def predict(self, model_name, candidates, method="scalar", use_prior=True):
         """
         Predict endpoint
-        :param model_name: The model path
-        :param candidates: A list of candidates
-        :return: list of predicted candidates as a map {property: [value, uncertainty]}
-        """
 
-        # If a single candidate is passed, wrap in a list for the user
-        if not isinstance(candidates, list):
-            candidates = [candidates]
+        :param model_name: The model identifier (id number for data views)
+        :param candidates: A list of candidates to make predictions on
+        :param method:     Method for propagating predictions through model graphs
+        :param use_prior:  Whether to apply prior values implied by the property descriptors
+        :return: the response, containing a list of predicted candidates as a map {property: [value, uncertainty]}
+        """
+        body = self._get_predict_body(candidates, method, use_prior)
 
         url = self._get_predict_url(model_name)
-        body = pif.dumps(
-            {"predictionRequest": {"predictionSource": "scalar", "usePrior": True, "candidates": candidates}}
-        )
+        response = self._post_with_version_check(url, data=body, headers=self.headers)
+        if response.status_code == 404:
+            url = self._get_deprecated_predict_url(model_name)
+            response = self._post_with_version_check(url, data=body, headers=self.headers)
 
-        response = requests.post(url, data=body, headers=self.headers)
         if response.status_code != requests.codes.ok:
             raise RuntimeError('Received ' + str(response.status_code) + ' response: ' + str(response.reason))
 
         return response.json()
 
+    def predict_custom(self, model_path, candidates):
+        """
+        Predict endpoint for a custom model
+
+        :param model_path: The path from the custom model url (https://citrination.com/predict/{{model_path}})
+        :param candidates: A list of candidates to make predictions on
+        :return: the response, containing a list of predicted candidates as a map {property: [value, uncertainty]}
+        """
+
+        body = self._get_predict_body(candidates)
+
+        url = self._get_custom_predict_url(model_path)
+        response = self._post_with_version_check(url, data=body, headers=self.headers)
+
+        if response.status_code != requests.codes.ok:
+            raise RuntimeError('Received ' + str(response.status_code) + ' response: ' + str(response.reason))
+
+        return response.json()
+
+    def _get_predict_body(self, candidates, method="scalar", use_prior=True):
+        if not (method == "scalar" or method == "from_distribution"):
+            raise ValueError("{} method not supported".format(method))
+
+        # If a single candidate is passed, wrap in a list for the user
+        if not isinstance(candidates, list):
+            candidates = [candidates]
+
+        return pif.dumps(
+             {"predictionRequest": {"predictionSource": method, "usePrior": use_prior, "candidates": candidates}}
+        )
+
+    def _get_custom_predict_url(self, model_path):
+        return self.api_url + '/ml_templates/' + model_path + '/predict'
+
     def _get_predict_url(self, model_name):
+        return self.api_url + '/data_views/' + model_name + '/predict'
+
+    def _get_deprecated_predict_url(self, model_name):
         return self.api_url + '/csv_to_models/' + model_name + '/predict'
 
-    def upload_file(self, file_path, data_set_id, root_path=None):
+    def upload(self, dataset_id, source_path, dest_path=None):
+        """
+        Upload a file, specifying source and dest paths a file (acts as the scp command)
+        :param source_path: The path to the file on the source host
+        :param dest_path: The path to the file where the contents of the upload will be written (on the dest host)
+        :return: A JSON block with a "message" key indicating that the upload if files is complete:
+                {
+                    "message": "Upload of files is complete.",
+                    "successes": List of filepaths which were uploaded successfully,
+                    "failures": List of filepaths which failed to upload
+                }
+        """
+        source_path = str(source_path)
+        if not dest_path:
+            dest_path = source_path
+        else:
+            dest_path = str(dest_path)
+
+        if os.path.isdir(source_path):
+            successes = []
+            failures = []
+            for path, subdirs, files in os.walk(source_path):
+                for name in files:
+                    path_without_root_dir = path.split("/")[1:] + [name]
+                    current_dest_path = os.path.join(dest_path, *path_without_root_dir)
+                    current_source_path = os.path.join(path, name)
+                    result = json.loads(self.upload(dataset_id, current_source_path, current_dest_path))
+                    if result["success"]:
+                        successes.append(current_source_path)
+                    else:
+                        failures.append(current_source_path)
+            message = {
+                "message": "Upload of files is complete.",
+                "successes": successes,
+                "failures": failures
+            }
+            return json.dumps(message)
+        elif os.path.isfile(source_path):
+            url = self._get_upload_url(dataset_id)
+            file_data = { "dest_path": str(dest_path), "src_path": str(source_path)}
+            r = self._post_with_version_check(url, data=json.dumps(file_data), headers=self.headers)
+            if r.status_code == 200:
+                j = json.loads(r.content.decode('utf-8'))
+                s3url = self._get_s3_presigned_url(j)
+                with open(source_path, 'rb') as f:
+                    r = requests.put(s3url, data=f, headers=j["required_headers"])
+                    if r.status_code == 200:
+                        url_data = {'s3object': j['url']['path'], 's3bucket': j['bucket']}
+                        self._post_with_version_check(self._get_update_file_upload_url(j['file_id']),
+                                      data=json.dumps(url_data), headers=self.headers)
+                        message = {"message": "Upload is complete.",
+                                   "data_set_id": str(dataset_id),
+                                   "version": j['dataset_version_id'],
+                                   "success": True}
+                        return json.dumps(message)
+                    else:
+                        message = {"message": "Upload failed.",
+                                   "status": r.status_code,
+                                   "success": False}
+                        return json.dumps(message)
+            else:
+                message = {"message": "Upload failed.",
+                                   "status": r.status_code,
+                                   "success": False}
+                return json.dumps(message)
+        else:
+            return json.dumps({
+                    "message": "No file at specified path {0}".format(source_path),
+                    "success": False
+                })
+
+    def upload_file(self, file_path, dataset_id, root_path=None):
         """
         Upload file to Citrination.
-
         :param file_path: File path to upload.
         :param data_set_id: The dataset id to upload the file to.
         :return: Response object or return code if the file was not uploaded.
         """
-        if os.path.isdir(str(file_path)):
-            for path, subdirs, files in os.walk(file_path):
-                for name in files:
-                    if root_path:
-                       root = root_path
-                    else:
-                       root = str(file_path)
-                    success = self.upload_file(os.path.join(path, name), data_set_id, root)
-                    print(success)
-            message = {"message": "Upload of files in " + str(root) + " is complete."}
-            return json.dumps(message)
+        return self.upload(dataset_id, file_path)
+
+    def _data_analysis(self, model_name):
+        """
+        Data analysis endpoint
+        :param model_name: The model identifier (id number for data views)
+        :return: dictionary containing information about the data, e.g. dCorr and tsne
+        """
+        url = self._get_data_analysis_url(model_name)
+        return self._get_content_from_url(url)
+
+    def tsne(self, model_name):
+        """
+        Get the t-SNE projection, including z-values and labels
+        :param model_name: The model identifier (id number for data views)
+        :return: dictionary containing property names and the projection for each
+        """
+        analysis = self._data_analysis(model_name)
+        projections = analysis['projections']
+        cleaned = {}
+        for k, v in projections.items():
+            d = {}
+            d['x'] = v['x']
+            d['y'] = v['y']
+            d['z'] = v['label']
+            d['label'] = v['inputs']
+            d['uid'] = v['uid']
+            cleaned[k] = d
+
+        return cleaned
+
+    def list_files(self, dataset_id, glob=".", is_dir=False):
+        """
+        List matched filenames in a dataset on Citrination.
+        :param dataset_id: The ID of the dataset to search for files.
+        :param glob: A pattern which will be matched against files in the dataset.
+        :param is_dir: A boolean indicating whether or not the pattern should match against the beginning of paths in the dataset.
+        :return: Response object or return code if the file was not uploaded.
+        """
+        url = self.api_url + '/datasets/' + str(dataset_id) + '/list_filepaths'
+        data = {
+            "list": {
+                "glob": glob,
+                "isDir": is_dir
+            }
+        }
+        r = self._post_with_version_check(url, data=json.dumps(data), headers=self.headers)
+        return json.loads(r.content.decode('utf-8'))
+
+    def matched_file_count(self, dataset_id, glob=".", is_dir=False):
+        list_result = self.list_files(dataset_id, glob, is_dir)
+        if isinstance(list_result["files"], list):
+            return len(list_result["files"])
         else:
-            url = self._get_upload_url(data_set_id)
-            file_data = {"file_path": str(file_path)}
-            if root_path:
-                file_data["root_path"] = root_path
+            return list_result
+
+    def get_matched_dataset_files(self, dataset_id, glob, is_dir=False, latest=True):
+        """
+        Retrieves URLs for the files matched by a glob or a path to a directory
+        in a given dataset.
+
+        :param dataset_id: The id of the dataset to retrieve files from
+        :param glob: A regex used to select one or more files in the dataset
+        :param is_dir: A flag used to indicate that the glob should be matched against the start of the paths in the dataset (simulates directory matching)
+        :param latest: A boolean flag indicating that results should be limited to reporting files from the latest dataset version
+        :return: The response object, or an error message object if the request failed
+        """
+        data = {
+            "download_request": {
+                "glob": glob,
+                "isDir": is_dir,
+                "latest": latest
+            }
+        }
+        url = self.api_url + '/datasets/' + str(dataset_id) + '/download_files'
+        r = self._post_with_version_check(url,data=json.dumps(data), headers=self.headers)
+        return json.loads(r.content.decode('utf-8'))
 
-            r = requests.post(url, data=json.dumps(file_data), headers=self.headers)
-            if r.status_code == 200:
-                j = json.loads(r.content.decode('utf-8'))
-                s3url = self._get_s3_presigned_url(j)
-                with open(file_path, 'rb') as f:
-                    r = requests.put(s3url, data=f)
-                    if r.status_code == 200:
-                        url_data = {'s3object': j['url']['path'], 's3bucket': j['bucket']}
-                        requests.post(self._get_update_file_upload_url(j['file_id']),
-                                      data=json.dumps(url_data), headers=self.headers)
-                        message = {"message": "Upload is complete.",
-                                   "data_set_id": str(data_set_id),
-                                   "version": j['dataset_version_id']}
-                        return json.dumps(message)
-                    else:
-                        message = {"message": "Upload failed.",
-                                   "status": r.status_code}
-                        return json.dumps(message)
-            else:
-                message = {"message": "Upload failed.",
-                                   "status": r.status_code}
-                return json.dumps(message)
 
     def get_dataset_files(self, dataset_id, latest = False):
         """
@@ -233,10 +435,8 @@ class CitrinationClient(object):
         :param latest: A boolean flag indicating that results should be limited to reporting files from the latest dataset version
         :return: The response object, or an error message object if the request failed
         """
-        if latest == False:
-            return self._get_content_from_url(self.api_url + '/data_sets/' + str(dataset_id) + '/files')
-        elif latest == True:
-            return self._get_content_from_url(self.api_url + '/data_sets/' + str(dataset_id) + '/latest')
+        if isinstance(latest, bool):
+            return self.get_matched_dataset_files(dataset_id, ".", False, latest)
         else:
             return {
                 "message": "If provided, the second parameter must be a boolean"
@@ -253,9 +453,9 @@ class CitrinationClient(object):
         :return: The response object, or an error message object if the request failed
         """
         if version == None:
-            return self._get_content_from_url(self.api_url + '/data_sets/' + str(dataset_id) + '/file/' + quote(file_path))
+            return self._get_content_from_url(self.api_url + '/datasets/' + str(dataset_id) + '/file/' + quote(file_path))
         else:
-            return self._get_content_from_url(self.api_url + '/data_sets/' + str(dataset_id) + '/version/' + str(version) + '/files/' + quote(file_path))
+            return self._get_content_from_url(self.api_url + '/datasets/' + str(dataset_id) + '/version/' + str(version) + '/files/' + quote(file_path))
 
     def get_pif(self, dataset_id, uid, version = None):
         """
@@ -294,6 +494,30 @@ class CitrinationClient(object):
         """
         return self.api_url+'/data_sets/'+str(data_set_id)+'/upload'
 
+    def _get_with_version_check(self, url, headers):
+        result = requests.get(url, headers=headers)
+        return self._check_response_for_version_mismatch(result)
+
+    def _post_with_version_check(self, url, data, headers):
+        result = requests.post(url, headers=headers, data=data)
+        return self._check_response_for_version_mismatch(result)
+
+    def _put_with_version_check(self, url, data, headers):
+        result = requests.put(url, data=data, headers=headers)
+        return self._check_response_for_version_mismatch(result)
+
+    def _check_response_for_version_mismatch(self, response):
+        response_content = json.loads(response.content.decode('utf-8'))
+        try:
+            if response.status_code == 400:
+                error_type = response_content["error_type"]
+                if error_type == "Version Mismatch":
+                    print("Version mismatch with Citrination identified. Please check for available PyCC updates")
+                    return False
+            return response
+        except KeyError:
+            return response
+
     @staticmethod
     def _get_s3_presigned_url(input_json):
         """
@@ -334,7 +558,7 @@ class CitrinationClient(object):
         if share:
             data["public"] = share
         dataset = {"dataset": data}
-        return requests.post(url, headers=self.headers, data=json.dumps(dataset))
+        return self._post_with_version_check(url, headers=self.headers, data=json.dumps(dataset))
 
     def _get_create_data_set_url(self):
         """
@@ -364,7 +588,7 @@ class CitrinationClient(object):
         if share:
             data["public"] = share
         dataset = {"dataset": data}
-        return requests.post(url, headers=self.headers, data=json.dumps(dataset))
+        return self._post_with_version_check(url, headers=self.headers, data=json.dumps(dataset))
 
     def _get_update_data_set_url(self, data_set_id):
         """
@@ -382,7 +606,7 @@ class CitrinationClient(object):
         :return: Response from the create data set version request.
         """
         url = self._get_create_data_set_version_url(data_set_id)
-        return requests.post(url, headers=self.headers)
+        return self._post_with_version_check(url, headers=self.headers, data=json.dumps({}))
 
     def _get_create_data_set_version_url(self, data_set_id):
         """
@@ -391,3 +615,6 @@ class CitrinationClient(object):
         :return: URL for creating new data set versions.
         """
         return self.api_url+'/data_sets/'+str(data_set_id)+'/create_dataset_version'
+
+    def _get_data_analysis_url(self, model_name):
+        return self.api_url + '/data_views/' + model_name + '/data_analysis'
