@@ -1,5 +1,6 @@
 from pypif.util.serializable import Serializable
-from citrination_client.search.pif.query.core.field_operation import FieldOperation
+
+from citrination_client.search.pif.query.core.field_query import FieldQuery
 
 
 class BaseObjectQuery(Serializable):
@@ -7,22 +8,25 @@ class BaseObjectQuery(Serializable):
     Base class for all PIF object queries.
     """
 
-    def __init__(self, logic=None, extract_as=None, extract_all=None, extract_when_missing=None, tags=None,
-                 length=None, offset=None):
+    def __init__(self, logic=None, simple=None, extract_as=None, extract_all=None, extract_when_missing=None, 
+                 tags=None, length=None, offset=None, **kwargs):
         """
         Constructor.
 
         :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
+        :param simple: String with the simple search to run against all fields.
         :param extract_as: String with the alias to save this field under.
         :param extract_all: Boolean setting whether all values in an array should be extracted.
         :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
         is missing that should be extracted (and the overall query is still satisfied).
-        :param tags: One or more :class:`FieldOperation` operations against the tags field.
-        :param length: One or more :class:`FieldOperation` operations against the length field.
-        :param offset: One or more :class:`FieldOperation` operations against the offset field.
+        :param tags: One or more :class:`FieldQuery` operations against the tags field.
+        :param length: One or more :class:`FieldQuery` operations against the length field.
+        :param offset: One or more :class:`FieldQuery` operations against the offset field.
         """
         self._logic = None
         self.logic = logic
+        self._simple = None
+        self.simple = simple
         self._extract_as = None
         self.extract_as = extract_as
         self._extract_all = None
@@ -48,6 +52,18 @@ class BaseObjectQuery(Serializable):
     def logic(self):
         self._logic = None
 
+    @property
+    def simple(self):
+        return self._simple
+
+    @simple.setter
+    def simple(self, simple):
+        self._simple = simple
+
+    @simple.deleter
+    def simple(self):
+        self._simple = None
+
     @property
     def extract_as(self):
         return self._extract_as
@@ -90,7 +106,7 @@ class BaseObjectQuery(Serializable):
 
     @tags.setter
     def tags(self, tags):
-        self._tags = self._get_object(FieldOperation, tags)
+        self._tags = self._get_object(FieldQuery, tags)
 
     @tags.deleter
     def tags(self):
@@ -102,7 +118,7 @@ class BaseObjectQuery(Serializable):
 
     @length.setter
     def length(self, length):
-        self._length = self._get_object(FieldOperation, length)
+        self._length = self._get_object(FieldQuery, length)
 
     @length.deleter
     def length(self):
@@ -114,7 +130,7 @@ class BaseObjectQuery(Serializable):
 
     @offset.setter
     def offset(self, offset):
-        self._offset = self._get_object(FieldOperation, offset)
+        self._offset = self._get_object(FieldQuery, offset)
 
     @offset.deleter
     def offset(self):
