@@ -1,5 +1,5 @@
 from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
-from citrination_client.search.pif.query.core.field_operation import FieldOperation
+from citrination_client.search.pif.query.core.field_query import FieldQuery
 from citrination_client.search.pif.query.core.value_query import ValueQuery
 
 
@@ -8,29 +8,33 @@ class ProcessStepQuery(BaseObjectQuery):
     Class to query against a process step.
     """
 
-    def __init__(self, name=None, details=None, logic=None, extract_as=None, extract_all=None,
-                 extract_when_missing=None, tags=None, length=None, offset=None):
+    def __init__(self, logic=None, simple=None, extract_as=None, extract_all=None, extract_when_missing=None, 
+                 tags=None, length=None, offset=None, name=None, details=None, query=None, **kwargs):
         """
         Constructor.
 
-        :param name: One or more :class:`FieldOperation` operations against the name field.
-        :param details: One or more :class:`ValueQuery` operations against the details of the step.
         :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
+        :param simple: String with the query to run against all fields.
         :param extract_as: String with the alias to save this field under.
         :param extract_all: Boolean setting whether all values in an array should be extracted.
         :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
         is missing that should be extracted (and the overall query is still satisfied).
-        :param tags: One or more :class:`FieldOperation` operations against the tags field.
-        :param length: One or more :class:`FieldOperation` operations against the length field.
-        :param offset: One or more :class:`FieldOperation` operations against the offset field.
+        :param tags: One or more :class:`FieldQuery` operations against the tags field.
+        :param length: One or more :class:`FieldQuery` operations against the length field.
+        :param offset: One or more :class:`FieldQuery` operations against the offset field.
+        :param name: One or more :class:`FieldQuery` operations against the name field.
+        :param details: One or more :class:`ValueQuery` operations against the details of the step.
+        :param query: One or more :class:`ProcessStepQuery` objects with nested queries.
         """
-        super(ProcessStepQuery, self).__init__(logic=logic, extract_as=extract_as, extract_all=extract_all,
-                                               extract_when_missing=extract_when_missing, tags=tags, length=length,
-                                               offset=offset)
+        super(ProcessStepQuery, self).__init__(
+            logic=logic, simple=simple, extract_as=extract_as, extract_all=extract_all,
+            extract_when_missing=extract_when_missing, tags=tags, length=length, offset=offset, **kwargs)
         self._name = None
         self.name = name
         self._details = None
         self.details = details
+        self._query = None
+        self.query = query
 
     @property
     def name(self):
@@ -38,7 +42,7 @@ class ProcessStepQuery(BaseObjectQuery):
 
     @name.setter
     def name(self, name):
-        self._name = self._get_object(FieldOperation, name)
+        self._name = self._get_object(FieldQuery, name)
 
     @name.deleter
     def name(self):
@@ -55,3 +59,15 @@ class ProcessStepQuery(BaseObjectQuery):
     @details.deleter
     def details(self):
         self._details = None
+
+    @property
+    def query(self):
+        return self._query
+
+    @query.setter
+    def query(self, query):
+        self._query = self._get_object(ProcessStepQuery, query)
+
+    @query.deleter
+    def query(self):
+        self._query = None
