@@ -1,5 +1,6 @@
 from citrination_client.search.pif.query.core.base_object_query import BaseObjectQuery
-from citrination_client.search.pif.query.core.field_operation import FieldOperation
+from citrination_client.search.pif.query.core.display_item_query import DisplayItemQuery
+from citrination_client.search.pif.query.core.field_query import FieldQuery
 from citrination_client.search.pif.query.core.name_query import NameQuery
 from citrination_client.search.pif.query.core.pages_query import PagesQuery
 
@@ -9,41 +10,46 @@ class ReferenceQuery(BaseObjectQuery):
     Class used to query against a PIF Reference object.
     """
 
-    def __init__(self, doi=None, isbn=None, issn=None, url=None, title=None, publisher=None, journal=None,
-                 volume=None, issue=None, year=None, pages=None, authors=None, editors=None, affiliations=None,
-                 acknowledgements=None, references=None, logic=None, extract_as=None, extract_all=None,
-                 extract_when_missing=None, tags=None, length=None, offset=None):
+    def __init__(self, logic=None, simple=None, extract_as=None, extract_all=None, extract_when_missing=None, 
+                 tags=None, length=None, offset=None, doi=None, isbn=None, issn=None, url=None, title=None, 
+                 publisher=None, journal=None, volume=None, issue=None, year=None, figure=None, table=None, 
+                 pages=None, authors=None, editors=None, affiliations=None, acknowledgements=None, references=None, 
+                 query=None, **kwargs):
         """
         Constructor.
 
-        :param doi: One or more :class:`FieldOperation` operations against the doi field.
-        :param isbn: One or more :class:`FieldOperation` operations against the isbn field.
-        :param issn: One or more :class:`FieldOperation` operations against the issn field.
-        :param url: One or more :class:`FieldOperation` operations against the url field.
-        :param title: One or more :class:`FieldOperation` operations against the title field.
-        :param publisher: One or more :class:`FieldOperation` operations against the publisher field.
-        :param journal: One or more :class:`FieldOperation` operations against the journal field.
-        :param volume: One or more :class:`FieldOperation` operations against the volume field.
-        :param issue: One or more :class:`FieldOperation` operations against the issue field.
-        :param year: One or more :class:`FieldOperation` operations against the year field.
-        :param pages: One or more :class:`PagesQuery` operations against the pages field.
-        :param authors: One or more :class:`NameQuery` operations against the authors field.
-        :param editors: One or more :class:`NameQuery` operations against the editors field.
-        :param affiliations: One or more :class:`FieldOperation` operations against the affiliations field.
-        :param acknowledgements: One or more :class:`FieldOperation` operations against the acknowledgements field.
-        :param references: One or more :class:`ReferenceQuery` operations against the references field.
         :param logic: Logic for this filter. Must be equal to one of "MUST", "MUST_NOT", "SHOULD", or "OPTIONAL".
+        :param simple: String with the simple query to run against all fields.
         :param extract_as: String with the alias to save this field under.
         :param extract_all: Boolean setting whether all values in an array should be extracted.
         :param extract_when_missing: Any valid JSON-supported object or PIF object. This value is returned when a value
         is missing that should be extracted (and the overall query is still satisfied).
-        :param tags: One or more :class:`FieldOperation` operations against the tags field.
-        :param length: One or more :class:`FieldOperation` operations against the length field.
-        :param offset: One or more :class:`FieldOperation` operations against the offset field.
+        :param tags: One or more :class:`FieldQuery` operations against the tags field.
+        :param length: One or more :class:`FieldQuery` operations against the length field.
+        :param offset: One or more :class:`FieldQuery` operations against the offset field.
+        :param doi: One or more :class:`FieldQuery` operations against the doi field.
+        :param isbn: One or more :class:`FieldQuery` operations against the isbn field.
+        :param issn: One or more :class:`FieldQuery` operations against the issn field.
+        :param url: One or more :class:`FieldQuery` operations against the url field.
+        :param title: One or more :class:`FieldQuery` operations against the title field.
+        :param publisher: One or more :class:`FieldQuery` operations against the publisher field.
+        :param journal: One or more :class:`FieldQuery` operations against the journal field.
+        :param volume: One or more :class:`FieldQuery` operations against the volume field.
+        :param issue: One or more :class:`FieldQuery` operations against the issue field.
+        :param year: One or more :class:`FieldQuery` operations against the year field.
+        :param figure: One or more :class:`DisplayItemQuery` operations against the figure field.
+        :param table: One or more :class:`DisplayItemQuery` operations against the table field.
+        :param pages: One or more :class:`PagesQuery` operations against the pages field.
+        :param authors: One or more :class:`NameQuery` operations against the authors field.
+        :param editors: One or more :class:`NameQuery` operations against the editors field.
+        :param affiliations: One or more :class:`FieldQuery` operations against the affiliations field.
+        :param acknowledgements: One or more :class:`FieldQuery` operations against the acknowledgements field.
+        :param references: One or more :class:`ReferenceQuery` operations against the references field.
+        :param query: One or more :class:`ReferenceQuery` objects with nested queries.
         """
-        super(ReferenceQuery, self).__init__(logic=logic, extract_as=extract_as, extract_all=extract_all,
-                                             extract_when_missing=extract_when_missing, tags=tags, length=length,
-                                             offset=offset)
+        super(ReferenceQuery, self).__init__(
+            logic=logic, simple=simple, extract_as=extract_as, extract_all=extract_all,
+            extract_when_missing=extract_when_missing, tags=tags, length=length, offset=offset, **kwargs)
         self._doi = None
         self.doi = doi
         self._isbn = None
@@ -64,6 +70,10 @@ class ReferenceQuery(BaseObjectQuery):
         self.issue = issue
         self._year = None
         self.year = year
+        self._figure = None
+        self.figure = figure
+        self._table = None
+        self.table = table
         self._pages = None
         self.pages = pages
         self._authors = None
@@ -76,6 +86,8 @@ class ReferenceQuery(BaseObjectQuery):
         self.acknowledgements = acknowledgements
         self._references = None
         self.references = references
+        self._query = None
+        self.query = query
 
     @property
     def doi(self):
@@ -83,7 +95,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @doi.setter
     def doi(self, doi):
-        self._doi = self._get_object(FieldOperation, doi)
+        self._doi = self._get_object(FieldQuery, doi)
 
     @doi.deleter
     def doi(self):
@@ -95,7 +107,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @isbn.setter
     def isbn(self, isbn):
-        self._isbn = self._get_object(FieldOperation, isbn)
+        self._isbn = self._get_object(FieldQuery, isbn)
 
     @isbn.deleter
     def isbn(self):
@@ -107,7 +119,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @issn.setter
     def issn(self, issn):
-        self._issn = self._get_object(FieldOperation, issn)
+        self._issn = self._get_object(FieldQuery, issn)
 
     @issn.deleter
     def issn(self):
@@ -119,7 +131,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @url.setter
     def url(self, url):
-        self._url = self._get_object(FieldOperation, url)
+        self._url = self._get_object(FieldQuery, url)
 
     @url.deleter
     def url(self):
@@ -131,7 +143,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @title.setter
     def title(self, title):
-        self._title = self._get_object(FieldOperation, title)
+        self._title = self._get_object(FieldQuery, title)
 
     @title.deleter
     def title(self):
@@ -143,7 +155,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @publisher.setter
     def publisher(self, publisher):
-        self._publisher = self._get_object(FieldOperation, publisher)
+        self._publisher = self._get_object(FieldQuery, publisher)
 
     @publisher.deleter
     def publisher(self):
@@ -155,7 +167,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @journal.setter
     def journal(self, journal):
-        self._journal = self._get_object(FieldOperation, journal)
+        self._journal = self._get_object(FieldQuery, journal)
 
     @journal.deleter
     def journal(self):
@@ -167,7 +179,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @volume.setter
     def volume(self, volume):
-        self._volume = self._get_object(FieldOperation, volume)
+        self._volume = self._get_object(FieldQuery, volume)
 
     @volume.deleter
     def volume(self):
@@ -179,7 +191,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @issue.setter
     def issue(self, issue):
-        self._issue = self._get_object(FieldOperation, issue)
+        self._issue = self._get_object(FieldQuery, issue)
 
     @issue.deleter
     def issue(self):
@@ -191,12 +203,36 @@ class ReferenceQuery(BaseObjectQuery):
 
     @year.setter
     def year(self, year):
-        self._year = self._get_object(FieldOperation, year)
+        self._year = self._get_object(FieldQuery, year)
 
     @year.deleter
     def year(self):
         self._year = None
 
+    @property
+    def figure(self):
+        return self._figure
+
+    @figure.setter
+    def figure(self, figure):
+        self._figure = self._get_object(DisplayItemQuery, figure)
+
+    @figure.deleter
+    def figure(self):
+        self._figure = None
+
+    @property
+    def table(self):
+        return self._table
+
+    @table.setter
+    def table(self, table):
+        self._table = self._get_object(DisplayItemQuery, table)
+
+    @table.deleter
+    def table(self):
+        self._table = None
+
     @property
     def pages(self):
         return self._pages
@@ -239,7 +275,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @affiliations.setter
     def affiliations(self, affiliations):
-        self._affiliations = self._get_object(FieldOperation, affiliations)
+        self._affiliations = self._get_object(FieldQuery, affiliations)
 
     @affiliations.deleter
     def affiliations(self):
@@ -251,7 +287,7 @@ class ReferenceQuery(BaseObjectQuery):
 
     @acknowledgements.setter
     def acknowledgements(self, acknowledgements):
-        self._acknowledgements = self._get_object(FieldOperation, acknowledgements)
+        self._acknowledgements = self._get_object(FieldQuery, acknowledgements)
 
     @acknowledgements.deleter
     def acknowledgements(self):
@@ -268,3 +304,15 @@ class ReferenceQuery(BaseObjectQuery):
     @references.deleter
     def references(self):
         self._references = None
+
+    @property
+    def query(self):
+        return self._query
+
+    @query.setter
+    def query(self, query):
+        self._query = self._get_object(ReferenceQuery, query)
+
+    @query.deleter
+    def query(self):
+        self._query = None
